/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "definitions.h"
#include "app.h"
#include "atf22lv10c.h"
#include <string.h>

#define FUSENUM 768
// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

/* Recieve data buffer */
uint8_t receiveDataBuffer[64] CACHE_ALIGN;

/* Transmit data buffer */
uint8_t  transmitDataBuffer[64] CACHE_ALIGN;

bool jedec;

extern uint8_t fuse[768];
extern uint8_t read_fuse[768];
extern uint8_t pes[12];

void Transmit_Armed(void);
// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

USB_DEVICE_HID_EVENT_RESPONSE APP_USBDeviceHIDEventHandler
(
    USB_DEVICE_HID_INDEX iHID,
    USB_DEVICE_HID_EVENT event,
    void * eventData,
    uintptr_t userData
)
{
    USB_DEVICE_HID_EVENT_DATA_REPORT_SENT * reportSent;
    USB_DEVICE_HID_EVENT_DATA_REPORT_RECEIVED * reportReceived;

    /* Check type of event */
    switch (event)
    {
        case USB_DEVICE_HID_EVENT_REPORT_SENT:

            /* The eventData parameter will be USB_DEVICE_HID_EVENT_REPORT_SENT
             * pointer type containing details about the report that was
             * sent. */
            reportSent = (USB_DEVICE_HID_EVENT_DATA_REPORT_SENT *) eventData;
            if(reportSent->handle == appData.txTransferHandle )
            {
                // Transfer progressed.
                appData.hidDataTransmitted = true;
            }
            
            break;

        case USB_DEVICE_HID_EVENT_REPORT_RECEIVED:

            /* The eventData parameter will be USB_DEVICE_HID_EVENT_REPORT_RECEIVED
             * pointer type containing details about the report that was
             * received. */

            reportReceived = (USB_DEVICE_HID_EVENT_DATA_REPORT_RECEIVED *) eventData;
            if(reportReceived->handle == appData.rxTransferHandle )
            {
                // Transfer progressed.
                appData.hidDataReceived = true;
            }
          
            break;

        case USB_DEVICE_HID_EVENT_SET_IDLE:

            /* For now we just accept this request as is. We acknowledge
             * this request using the USB_DEVICE_HID_ControlStatus()
             * function with a USB_DEVICE_CONTROL_STATUS_OK flag */

            USB_DEVICE_ControlStatus(appData.usbDevHandle, USB_DEVICE_CONTROL_STATUS_OK);

            /* Save Idle rate recieved from Host */
            appData.idleRate = ((USB_DEVICE_HID_EVENT_DATA_SET_IDLE*)eventData)->duration;
            break;

        case USB_DEVICE_HID_EVENT_GET_IDLE:

            /* Host is requesting for Idle rate. Now send the Idle rate */
            USB_DEVICE_ControlSend(appData.usbDevHandle, & (appData.idleRate),1);

            /* On successfully reciveing Idle rate, the Host would acknowledge back with a
               Zero Length packet. The HID function drvier returns an event
               USB_DEVICE_HID_EVENT_CONTROL_TRANSFER_DATA_SENT to the application upon
               receiving this Zero Length packet from Host.
               USB_DEVICE_HID_EVENT_CONTROL_TRANSFER_DATA_SENT event indicates this control transfer
               event is complete */

            break;
        default:
            // Nothing to do.
            break;
    }
    return USB_DEVICE_HID_EVENT_RESPONSE_NONE;
}

void APP_USBDeviceEventHandler(USB_DEVICE_EVENT event, void * eventData, uintptr_t context)
{
    switch(event)
    {
        case USB_DEVICE_EVENT_RESET:
        case USB_DEVICE_EVENT_DECONFIGURED:

            /* Host has de configured the device or a bus reset has happened.
             * Device layer is going to de-initialize all function drivers.
             * Hence close handles to all function drivers (Only if they are
             * opened previously. */

            appData.deviceConfigured = false;
            appData.state = APP_STATE_WAIT_FOR_CONFIGURATION;
            
            break;

        case USB_DEVICE_EVENT_CONFIGURED:

            /* Set the flag indicating device is configured. */
            appData.deviceConfigured = true;

            /* Save the other details for later use. */
            appData.configurationValue = ((USB_DEVICE_EVENT_DATA_CONFIGURED*)eventData)->configurationValue;

            /* Register application HID event handler */
            USB_DEVICE_HID_EventHandlerSet(USB_DEVICE_HID_INDEX_0, APP_USBDeviceHIDEventHandler, (uintptr_t)&appData);
            
            break;

        case USB_DEVICE_EVENT_SUSPENDED:

            break;

        case USB_DEVICE_EVENT_POWER_DETECTED:

            /* VBUS was detected. We can attach the device */

            USB_DEVICE_Attach (appData.usbDevHandle);
            break;

        case USB_DEVICE_EVENT_POWER_REMOVED:

            /* VBUS is not available */
            USB_DEVICE_Detach(appData.usbDevHandle);
            break;

        /* These events are not used in this demo */
        case USB_DEVICE_EVENT_RESUMED:
        case USB_DEVICE_EVENT_ERROR:
        default:
            break;
    }
}

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;
    
    appData.usbDevHandle = USB_DEVICE_HANDLE_INVALID;
    appData.deviceConfigured = false;
    appData.txTransferHandle = USB_DEVICE_HID_TRANSFER_HANDLE_INVALID;
    appData.rxTransferHandle = USB_DEVICE_HID_TRANSFER_HANDLE_INVALID;
    appData.hidDataReceived = false;
    appData.hidDataTransmitted = true;
    appData.receiveDataBuffer = &receiveDataBuffer[0];
    appData.transmitDataBuffer = &transmitDataBuffer[0];
    
    TurnOff();

}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{
    static uint16_t i,j;
    
    /* Check if device is configured.  See if it is configured with correct
     * configuration value  */

    switch(appData.state)
    {
        /* Application's initial state. */
        case APP_STATE_INIT:

            /* Open the device layer */
            appData.usbDevHandle = USB_DEVICE_Open( USB_DEVICE_INDEX_0, DRV_IO_INTENT_READWRITE );

            if(appData.usbDevHandle != USB_DEVICE_HANDLE_INVALID)
            {
                /* Register a callback with device layer to get event notification (for end point 0) */
                USB_DEVICE_EventHandlerSet(appData.usbDevHandle, APP_USBDeviceEventHandler, 0);

                appData.state = APP_STATE_WAIT_FOR_CONFIGURATION;
            }
            else
            {
                /* The Device Layer is not ready to be opened. We should try
                 * again later. */
            }

            break;

        case APP_STATE_WAIT_FOR_CONFIGURATION:

            if(appData.deviceConfigured == true)
            {
                /* Device is ready to run the main task */
                appData.hidDataReceived = false;
                appData.hidDataTransmitted = true;
                appData.state = APP_STATE_MAIN_TASK;
                LED1_Set();
    
                /* Place a new read request. */
                USB_DEVICE_HID_ReportReceive (USB_DEVICE_HID_INDEX_0,
                        &appData.rxTransferHandle, appData.receiveDataBuffer, 64);
            }
            break;

        case APP_STATE_MAIN_TASK:

            if(!appData.deviceConfigured)
            {
                /* Device is not configured */
                appData.state = APP_STATE_WAIT_FOR_CONFIGURATION;

            } else if (appData.hidDataReceived && appData.hidDataTransmitted) {               
                
                    if(appData.receiveDataBuffer[0] == 'j'){
                        j = 0;
                        for(i=0; i<FUSENUM; i++) read_fuse[i]=0;           
                        appData.transmitDataBuffer[0] = 'a';
      
                        Transmit_Armed();
                        appData.state = APP_STATE_JEDEC;
                        break;
                    }
                    else if(appData.receiveDataBuffer[0] == 'p'){
                        appData.state = APP_STATE_PES;
                        break;
                    }else if(appData.receiveDataBuffer[0] == 'w'){
                        appData.state = APP_STATE_WRITE;
                        break;
                    }else if(appData.receiveDataBuffer[0] == 'r'){
                        j = 0;
                        appData.state = APP_STATE_READ;

                        LED1_Clear();              
                        ReadGAL();
                        LED1_Set();
                        
                        break;
                    }
            }
            break;   
                
        case APP_STATE_JEDEC:
            if(!appData.deviceConfigured){
                /* Device is not configured */
                appData.state = APP_STATE_WAIT_FOR_CONFIGURATION;

            } else if (appData.hidDataReceived && appData.hidDataTransmitted) { 
                for(i=0; i<64; i++) fuse[i+j*64]= appData.receiveDataBuffer[i];
            
                appData.transmitDataBuffer[0] = 'a';
                
                Transmit_Armed();
                if(++j == 12) appData.state = APP_STATE_MAIN_TASK;
            }
            break;
            
        case APP_STATE_PES:          
            
            if(!appData.deviceConfigured){
                /* Device is not configured */
                appData.state = APP_STATE_WAIT_FOR_CONFIGURATION;

            } else if (appData.hidDataReceived && appData.hidDataTransmitted) {
                LED1_Clear();
                TurnOn();
                GetPES(pes);
                TurnOff();
                LED1_Set();
                for(i=0;i<9;i++) appData.transmitDataBuffer[i]=pes[i]; 
                
                Transmit_Armed();       
                appData.state = APP_STATE_MAIN_TASK;      
            }
            
            break;
        
        case APP_STATE_WRITE:
             if(!appData.deviceConfigured){
                /* Device is not configured */
                appData.state = APP_STATE_WAIT_FOR_CONFIGURATION;
            } else if (appData.hidDataReceived && appData.hidDataTransmitted) {           
            LED1_Clear();             
            EraseGAL();
            WriteGAL();
            ReadGAL();
            LED1_Set();
            
            if(CompFuse()) appData.transmitDataBuffer[0] = 'a';
            else appData.transmitDataBuffer[0] = 'n';
            
            Transmit_Armed();
            appData.state = APP_STATE_MAIN_TASK;
            }   
            break;        
            
        case APP_STATE_READ:
             if(!appData.deviceConfigured){
                /* Device is not configured */
                appData.state = APP_STATE_WAIT_FOR_CONFIGURATION;
            } else if (appData.hidDataReceived && appData.hidDataTransmitted) {
                
                for(i=0;i<64;i++) appData.transmitDataBuffer[i]=read_fuse[i+64*j];
                
                Transmit_Armed();
                if(++j == 12) appData.state = APP_STATE_MAIN_TASK;
            }
             break;

        case APP_STATE_ERROR: 
            
            break;                
  
        default:
            break;
    }
}

void Transmit_Armed(void){
    appData.hidDataTransmitted = false;
    USB_DEVICE_HID_ReportSend(USB_DEVICE_HID_INDEX_0,
            &appData.txTransferHandle, appData.transmitDataBuffer, 64);
                
    appData.hidDataReceived = false;
    USB_DEVICE_HID_ReportReceive(USB_DEVICE_HID_INDEX_0,
            &appData.rxTransferHandle, appData.receiveDataBuffer, 64);
}

/*******************************************************************************
 End of File
 */
