/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <xc.h>
#include <stdbool.h>
#include <stdint.h>
#include "atf22lv10c.h"
#include "definitions.h"
        
#define ON true
#define OFF false
#define HIGH true
#define LOW false
#define LASTFUSE 5892

#define UES_FUSE 5828
#define UES_ROW 44

#define FUSENUM 768
/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

uint8_t pes[12];
uint8_t fuse[FUSENUM];
uint8_t read_fuse[FUSENUM];

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

uint16_t progtime=10;

// configを読み書きするアドレスの順番
uint16_t cfg22V10[]=
{
    5809,5808,
    5811,5810,
    5813,5812,
    5815,5814,
    5817,5816,
    5819,5818,
    5821,5820,
    5823,5822,
    5825,5824,
    5827,5826
};

 //ATF 2番（VPP）ピンにプログラム用電圧を掛けるか切るかの設定
void SetVPP(bool on)
{
    if (on == ON) VPP_Clear();  // VPP on
    else VPP_Set(); // VPP off
}

//ATF 3番（P/V）ピンの設定　H:プログラム　L:べリファイ（検証）
void SetPV(bool on)
{
    if (on) P_V_Set(); // Program P/V
    else    P_V_Clear(); // Verify
}

// 書き込み、読み込みなどの状態設定
// ATF （4,5,6,7,9）番ピンの組（外部で互いに接続してある）と8番ピンの状態設定
// rowの0ビット目は（4,5,6,7,9）番ピンの組の設定、1ビット目は8番ピンの組の設定
// 次の３つのモードがある
// row == 0x00の場合：コンフィーグ以外のフューズの読み書き
// row == 0x02の場合：コンフィーグ・フューズの読み書き
// row == 0x03の場合：通常状態、ATFを消去する場合もこのモード
void SetRow(uint8_t row)
{   
    if((row & 0x01) == 0x01) ERASE_Set();
    else ERASE_Clear();
    if((row & 0x02) ==0x02)  DATA_SELECT_Set();
    else DATA_SELECT_Clear();
}

//ATF 10番（SCLK）ピン シリアルクロック（serial clock）
void SetSCLK(bool on)
{
    if(on) CLK_Set();
    else CLK_Clear();
}

//ATF 11番（SDIN）ピン シリアルデータ入力（serial data input）
void SetSDIN(bool on)
{
    if(on) SDIN_Set();
    else SDIN_Clear();
}

//ATF 13番（STB）ピン ストローブ（strobe）
void SetSTB(bool on)
{
    if(on) STB_Set();
    else STB_Clear();
}

//**************************************************************

/////////////////////////////////////////////////////////////////
bool TurnOn(void){
     
	SetVPP(OFF);    //VPP (12V) off
	SetPV(LOW);     // P/V- low
	SetRow(0x03);   // SetRowのコメントを読むこと
	SetSDIN(HIGH);  // SDIN high
	SetSCLK(HIGH);  // SCLK high
	SetSTB(HIGH);   // STB high
	CORETIMER_DelayMs(100);
	SetSCLK(LOW);   // SCLK low

	SetVPP(ON); // VPP on
	CORETIMER_DelayMs(20);
    
	return true;
}

void TurnOff(void)
{
    CORETIMER_DelayMs(100);
    SetPV(LOW);     // P/V- low
    SetRow(0x03);   // SetRowのコメントを読むこと
    SetSDIN(HIGH);  // SDIN high
    SetVPP(OFF);    // Vpp off (+12V)
    SetPV(HIGH);    // P/V- high
    CORETIMER_DelayMs(2);
}

void StrobeRow(uint8_t row){
    
    SetRow(0x00);       // SetRowのコメントを読むこと
    SendAddress(row);   // rowの番号(6 bits)を送り込む
    SetSTB(LOW);
CORETIMER_DelayUs(10);
    SetSTB(HIGH);       // パルス /STB
    SetSDIN(LOW);       // SDIN low
}

//　SDINへのクロックビット
void SendBit(bool bit1)
{
    SetSDIN(bit1);
    SetSCLK(HIGH);
CORETIMER_DelayUs(10);
    SetSCLK(LOW);

}

// rowの番号(6 bits)をSDINピンに送り込む
// MSB から　LSB へ
void SendAddress( uint8_t row)
{
    uint8_t n=6;
    
    while(n-- >1)
    {
        SendBit((bool)((row & 0x20) != 0)); // clock in row number bits 5-1
        row<<=1;
    }
       SetSDIN((bool)((row & 0x20) != 0));  // SDIN = row number bit 0
}

bool GetSDOUT(void)
{
//    int i;
//    i=PORTBbits.RB7;
    if(PORTBbits.RB7 == 0) return false;
    else return true;
}

//　SDOUTへのクロックビット
bool ReceiveBit(void)
{
    bool bit1;
    bit1=GetSDOUT();
    SetSCLK(HIGH);
CORETIMER_DelayUs(10);
    SetSCLK(LOW);
    return bit1;
}

void GetPES(uint8_t pes[])
{
    uint16_t bitmask;
    uint8_t byte;

    StrobeRow(58);
    for(byte=0;byte<10;byte++)
        {
        pes[byte]=0;
        for(bitmask=0x1; bitmask<=0x80; bitmask<<=1)
            {
            if(ReceiveBit()) pes[byte]|=bitmask;
        }
    }
}

//////////////////////////////////////////////////
//　ストローブ・パルス　ここでは、パルス幅を10msecとしている
void Strobe(uint16_t msec)
{
    SetSTB(0);
    CORETIMER_DelayMs(msec);
    SetSTB(1);
}

void EraseGAL(void){
    TurnOn();
    SetPV(HIGH);
    SetRow(0x03);   // SetRowのコメントを読むこと
    Strobe(10);     //消去に必要なパルス幅を10msecとしている
    SetPV(LOW);
    TurnOff();
}

////////////////////////////////////////////
void SendBits(uint16_t n, bool bit1)
{
    while(n-- >0) SendBit(bit1);
}

bool GetFuseBit(uint16_t num) {

	uint16_t i;
	uint8_t j, k;

	i = num / 8;
	j = num % 8;
	k = (uint8_t)(0x80 >> j);

	if ((fuse[i] & k) != 0) return true;
	else return false;

}

void WriteGAL(){
    
    uint16_t row,bit1;
 
    TurnOn();
        SetRow(0x00);   // SetRowのコメントを読むこと

    // フュ-ズの書き込み    
    for(row=0;row<44;row++)
    {
        for(bit1=0; bit1<132; bit1++)
        {
            
            SendBit(GetFuseBit(44*bit1+row));
        }
        SendAddress(row);  // rowの番号(6 bits)を送り込む
        SetPV(HIGH);
        Strobe(progtime);
        SetPV(LOW);
    }
   
    // UESの書き込み（132ビット）
    SendBits(68, true);//68ビット分、1で詰める 
    for(bit1=0; bit1<64; bit1++)
    {
        SendBit(GetFuseBit(UES_FUSE+bit1));
    }
    SendAddress(UES_ROW);    // rowの番号(6 bits)を送り込む
    SetPV(HIGH);
    Strobe(progtime);
    SetPV(LOW);
            
    // CFGの書き込み（20ビット）
    SetRow(0x02);   // SetRowのコメントを読むこと
    for(bit1=0; bit1<19; bit1++)
    {
        SendBit(GetFuseBit(cfg22V10[bit1]));    // 0から18ビット目を送り出す
    }
    SetSDIN(GetFuseBit(cfg22V10[19]));  // 19ビット目を送り出す
    SetPV(HIGH);
    Strobe(progtime);
    SetPV(LOW);
    
    // パワーダウン機能を無効にする (JEDEC bit #5892)
    SetRow(0x00);   // SetRowのコメントを読むこと
    SendAddress(59);    // rowの番号(6 bits)を送り込む
    SetPV(HIGH);
    Strobe(progtime);
    SetPV(LOW);

    TurnOff();
}

//////////////////////////////////////////////////////////
void SetFuseBit(uint16_t num) {

	uint16_t i;
	uint8_t j;

	i = num / 8;
	j = num % 8;

	read_fuse[i] |= (uint8_t)(0x80>>j);

}

void DiscardBits(int n)
{
    while(n-- >0) ReceiveBit();
}

void ReadGAL(void)
{
    uint16_t row, bit1,i;

    for(i=0;i<FUSENUM;i++) read_fuse[i]=0;
    
    TurnOn();
    
    // フュ-ズの読み込み
    for(row=0; row<44; row++)
    {
        StrobeRow(row);
        for(bit1=0; bit1<132; bit1++)
        {
            if(ReceiveBit()) SetFuseBit(44*bit1+row);
        }
       CORETIMER_DelayMs(1);
    }
    
    // UESの読み込み（132ビット）
    StrobeRow(UES_ROW);
    DiscardBits(68);//空読みする
    for(bit1=0; bit1<64; bit1++)
    {
        if(ReceiveBit()) SetFuseBit(UES_FUSE+bit1);
    }    
    CORETIMER_DelayMs(1);
        
    // CFGの読み込み（20ビット）
    SetRow(0x02);   // SetRowのコメントを読むこと
    Strobe(HIGH);
    for(bit1=0; bit1<20; bit1++)
    { 
        if(ReceiveBit()) SetFuseBit(cfg22V10[bit1]);
    }

    TurnOff();
}

bool CompFuse(void){
    uint16_t i;
    
    for(i=0; i<FUSENUM; i++)
        if(fuse[i]!=read_fuse[i]) return false;
    return true;
}
/* *****************************************************************************
 End of File
 */
